import json, click
import numpy as np

import super_utils as sutils
from rasterio import features, Affine
import mercantile


def union(inputtiles, parsenames):

    tiles = sutils.tile_parser(inputtiles, parsenames)

    xmin, xmax, ymin, ymax = sutils.get_range(tiles)

    zoom = sutils.get_zoom(tiles)

    # make an array of shape (xrange + 3, yrange + 3)
    burn = sutils.burnXYZs(tiles, xmin, xmax, ymin, ymax, 0)

    nw = mercantile.xy(*mercantile.ul(xmin, ymin, zoom))

    se = mercantile.xy(*mercantile.ul(xmax + 1, ymax + 1, zoom))

    aff = Affine(((se[0] - nw[0]) / float(xmax - xmin + 1)), 0.0, nw[0],
        0.0, -((nw[1] - se[1]) / float(ymax - ymin + 1)), nw[1])

    unprojecter = sutils.Unprojecter()

    unionedTiles = [
        {
            'geometry': unprojecter.unproject(feature),
            'properties': {},
            'type': 'Feature'
        } for feature, shapes in features.shapes(np.asarray(np.flipud(np.rot90(burn)).astype(np.uint8), order='C'), transform=aff) if shapes == 1
    ]

    return unionedTiles


if __name__ == '__main__':
    union()




import super_utils as sutils
import numpy as np
import mercantile
from affine import Affine

from rasterio import features

def project_geom(geom):
    return {
        'type': geom['type'],
        'coordinates': [
            [mercantile.xy(*coords) for coords in part]
            for part in geom['coordinates']
        ]
    }


def find_extrema(features):
    epsilon = 1.0e-10

    totalArr = np.array([c for f in features for poly in f['geometry']['coordinates'] for c in poly for poly in f])

    xMax = totalArr[:, 0].max() + epsilon
    xMin = totalArr[:, 0].min() - epsilon
    yMax = totalArr[:, 1].max() + epsilon
    yMin = totalArr[:, 1].min() - epsilon

    return [xMin, yMin, xMax, yMax]

def tile_extrema(bounds, zoom):
    minimumTile = mercantile.tile(bounds[0], bounds[3], zoom)
    maximumTile = mercantile.tile(bounds[2], bounds[1], zoom)

    return {
        'x': {
            'min': minimumTile.x,
            'max': maximumTile.x + 1
            },
        'y': {
            'min': minimumTile.y,
            'max': maximumTile.y + 1
            }
        }

def make_transform(tilerange, zoom):
    ulx, uly = mercantile.xy(*mercantile.ul(tilerange['x']['min'], tilerange['y']['min'], zoom))
    lrx, lry = mercantile.xy(*mercantile.ul(tilerange['x']['max'], tilerange['y']['max'], zoom))
    xcell = (lrx - ulx) / float(tilerange['x']['max'] - tilerange['x']['min'])
    ycell = (uly - lry) / float(tilerange['y']['max'] - tilerange['y']['min'])
    return Affine(xcell, 0, ulx,
        0, -ycell, uly)


def burn(polys, zoom):
    bounds = find_extrema(polys)
    tilerange = tile_extrema(bounds, zoom)
    afftrans = make_transform(tilerange, zoom)

    burn = features.rasterize(
            ((project_geom(geom['geometry']), 255) for geom in polys),
            out_shape=((tilerange['y']['max'] - tilerange['y']['min'], tilerange['x']['max'] - tilerange['x']['min'])),
            transform=afftrans,
            all_touched=True)

    xys = np.fliplr(np.dstack(np.where(burn))[0])

    xys[:, 0] += tilerange['x']['min']
    xys[:, 1] += tilerange['y']['min']

    return np.append(xys, np.zeros((xys.shape[0], 1), dtype=np.uint8) + zoom, axis=1)




import click, json
import cligj
from supermercado import edge_finder, uniontiles, burntiles, super_utils


@click.group('supermercado')
def cli():
    pass

@click.command('edges')
@click.argument('inputtiles', default='-', required=False)
@click.option('--parsenames', is_flag=True)
def edges(inputtiles, parsenames):
    """
    For a stream of [<x>, <y>, <z>] tiles, return only those tiles that are on the edge.
    """
    try:
        inputtiles = click.open_file(inputtiles).readlines()
    except IOError:
        inputtiles = [inputtiles]

    # parse the input stream into an array
    tiles = edge_finder.findedges(inputtiles, parsenames)

    for t in tiles:
        click.echo(t.tolist())

cli.add_command(edges)

@click.command('union')
@click.argument('inputtiles', default='-', required=False)
@click.option('--parsenames', is_flag=True)
def union(inputtiles, parsenames):
    """
    Returns the unioned shape of a steeam of [<x>, <y>, <z>] tiles in GeoJSON.
    """
    try:
        inputtiles = click.open_file(inputtiles).readlines()
    except IOError:
        inputtiles = [inputtiles]
    unioned = uniontiles.union(inputtiles, parsenames)
    for u in unioned:
        click.echo(json.dumps(u))

cli.add_command(union)


@click.command('burn')
@cligj.features_in_arg
@cligj.sequence_opt
@click.argument('zoom', type=int)
def burn(features, sequence, zoom):
    """
    Burn a stream of GeoJSONs into a output stream of the tiles they intersect for a given zoom.
    """
    features = [f for f in super_utils.filter_polygons(features)]

    tiles = burntiles.burn(features, zoom)
    for t in tiles:
        click.echo(t.tolist())


cli.add_command(burn)


import numpy as np

import json, re, mercantile

def parseString(tilestring, matcher):
    tile = [int(r) for r in matcher.match(tilestring).group().split('-')]
    tile.append(tile.pop(0))
    return tile

def get_range(xyz):
    return xyz[:, 0].min(), xyz[:, 0].max(), xyz[:, 1].min(), xyz[:, 1].max()

def burnXYZs(tiles, xmin, xmax, ymin, ymax, pad=1):
    # make an array of shape (xrange + 3, yrange + 3)
    burn = np.zeros((xmax - xmin + (pad * 2 + 1), ymax - ymin + (pad * 2 + 1)), dtype=bool)

    # using the tile xys as indicides, burn in True where a tile exists
    burn[(tiles[:,0] - xmin + pad, tiles[:, 1] - ymin + pad)] = True

    return burn


def tile_parser(tiles, parsenames=False):
    if parsenames:
        tMatch = re.compile(r"[\d]+-[\d]+-[\d]+")
        tiles = np.array([parseString(t, tMatch) for t in tiles])
    else:
        tiles = np.array([json.loads(t) for t in tiles])

    return tiles

def get_idx():
    tt = np.zeros((3, 3), dtype=bool)
    tt[1, 1] = True

    return np.dstack(np.where(tt != True))[0] - 1

def get_zoom(tiles):
    t, d = tiles.shape
    if t < 1 or d != 3:
        raise ValueError("Tiles must be of shape n, 3")

    if tiles[:, 2].min() != tiles[:, 2].max():
        raise ValueError("All tile zooms must be the same")

    return tiles[0, 2]

def filter_polygons(features):
    for f in features:
        if 'geometry' in f and 'type' in f['geometry']:
            if f['geometry']['type'] == 'Polygon':
                yield f
            elif f['geometry']['type'] == 'MultiPolygon':
                for part in f['geometry']['coordinates']:
                    yield {
                        'geometry': {
                            'type': 'Polygon',
                            'coordinates': part
                        }
                    }

class Unprojecter:
    def __init__(self):
        self.R2D = 180 / np.pi
        self.A = 6378137.0

    def xy_to_lng_lat(self, coordinates):
        for c in coordinates:
            tc = np.array(c)
            yield np.dstack([
                tc[:, 0] * self.R2D / self.A,
                ((np.pi * 0.5) - 2.0 * np.arctan(np.exp(-tc[:, 1] / self.A))) * self.R2D
                ])[0].tolist()

    def unproject(self, feature):
        feature['coordinates'] = [f for f in self.xy_to_lng_lat(feature['coordinates'])]
        return feature
